#!/bin/sh

# NAME:
#	gend - generic daemon script
#
# SYNOPSIS:
#	gend [-options]
#
# DESCRIPTION:
#	This is a generic script for a daemon service.
#	It is designed to be started regularly from cron(8), to ensure
#	it is running when needed.
#
#	It first checks that it is not already running.
#	Then it looks for "Myname".sub (where "Myname" is the name
#	the script was invoked as).  Which can provide replacement
#	functions for SetUp(), MainLoop(), CleanUp() and even Logger().
#
#	MainLoop() is the bulk of the daemon, and may well want to call
#	a HouseKeep() function to keep things tidy, the default
#	function uses '/etc/rc.sh' to process the directories named by
#	"SUBDIR" (default 'daily', 'loop') with '.d:S:start' appended.
#	This causes each file in 'daily.d' or 'loop.d' that starts with
#	an 'S' to be run with the argument 'start'.
#
#	While it is running, if the file 'stop' appears in the current
#	directory it will log the fact and terminate.
#
#	Similarly, if the file 'restart' appears the daemon will re-exec
#	itself.  This is mainly to ensure that software updates can be
#	automated.
#	 
#	Finally, if the file 'lock' appears it will stop processing
#	until the file is removed but otherwise keep running.
#
#	Options:
#
#	The options supported vary on "Myname".sub (by setting
#	"opt_str").  The following options are reserved:
#
#	-L "facility"
#		Log to syslog using "facility" (local1).
#
#	-I "interval"
#		"interval" is a schedule to keep to. We will
#		sleep long enough to keep to it.
#
#	-S "secs"
#		Sleep "secs" between interations (60), only
#		used if "interval" is not set.
#
#	-1	Loop only once.  Mainly for testing.
#		  
# SEE ALSO:
#	setopts.sh, source.sh, only1.sh
#
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>
#

# RCSid:
#	$Id: gend.sh,v 1.30 2023/03/22 02:49:55 sjg Exp $
#
#	@(#) Copyright (c) 1995-2022 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

Myname=`basename $0 .sh`
Mydir=`dirname $0`
case $Mydir in
/*)	;;
.)	Mydir=`pwd`;;
*)	Mydir=`cd $Mydir; pwd`;;
esac

# ensure we can restart ok
_restart_dir=${_restart_dir:-`pwd`}
_restart_cmd=${_restart_cmd:-"$_shell $0 $@"}
_restart_path=${_restart_path:-$PATH}

PATH=$Mydir:$_restart_path
export PATH _restart_dir _restart_cmd _restart_path

. source.sh

# pick this up asap to get extra PATHS
rc=.${Myname}rc
dot $Mydir/$rc

# this must exist
. only1.sh

for m in /usr/bin/Mail /usr/bin/mailx /usr/ucb/Mail
do
	test -x $m && { MAILER=$m; break; }
done
MAILER=${MAILER:-Mail}

opt_L=local1

case `echo -n .` in -n*) echo_n=; echo_c="\c";; *) echo_n=-n; echo_c=;; esac

Echo() {
	case "$1" in
	-n) shift;  echo $echo_n "$@$echo_c";;
	*) echo "$@";;
	esac
}

Max() {
	test $1 -gt $2 && echo $1 || echo $2
}

Min() {
	test $1 -lt $2 && echo $1 || echo $2
}

# functions that $Myname.sub can replace
SetUp() {
	SUBDIR="daily loop"
}

Restart() {
	rm -f restart
	cd $_restart_dir && exec $_restart_cmda
}

CheckStops() {
	if test -f stop; then
		Logger notice "stop file present, exiting..."
		CleanUp
		exit 0
	fi
	if test -f restart; then
		Logger info "restart file present, re-running $_restart_cmd"
		Restart
	fi
	if test -f lock; then
		sleep 10
		return 1
	fi
	return 0
}

trap CheckStops 1

MainLoop() {
	if test -x /etc/rc.sh; then
		for d in $SUBDIR
		do
			/etc/rc.sh $d.d:S:start
		done
	fi
}

CleanUp() {
	:
}

Logger() {
	lev=$1; shift
	echo "$*" | logger -t "${LOGGER_TAG:-$Myname[$$]}" -p $opt_L.$lev 
}

Secs() {
	date +%s
}

# pick up junk specific to this daemon,
# must include a function called MainLoop()
dot $Mydir/$Myname.sub
# might have a local version too.
dot ./$Myname.sub

# now re-load $rc before 
dot $Mydir/$rc

opt_str="I:L:S:1$opt_str"

# this must be direct
. setopts.sh

trap "" 1 2

SetUp

# log the fact that we were run.

Logger info ${_re}started.

# if we come past the previous line again, we were restarted.
_re=re
export _re

# this is the main loop

SECS=${SECS:-`Which secs`}
SECS=${SECS:-'Secs'}

rm -f restart

while :
do
	CheckStops || continue

	_start=`$SECS`

	MainLoop

	test x$opt_1 = x || exit 0
	
	test -f stop -o -f restart -o -f lock && continue

	if test "$opt_I"; then
		_end=`$SECS`
		_secs=`expr $opt_I + ${_start:-0} - ${_end:-0}`
		test ${_secs:-0} -gt 0 && sleep $_secs
	else
		sleep ${opt_S:-60}
	fi
done
# not normally reached...
CleanUp
